<?php

namespace FiloBlu\Flow\Cron;

/**
 * Class Fetch
 * @package FiloBlu\Flow\Cron
 */
class Fetch
{
    /**
     * @var \FiloBlu\Flow\Model\Processor
     */
    protected $_processor;

    /**
    * @var \Monolog\Logger
    */
    protected $_logger;

    protected $old_wait_timeout;
    protected $old_interactive_timeout;
    //to prevent 'General error: 2006 MySQL server has gone away ' error.
    protected $new_wait_value_timeout = 600; //10 Minutes

    /**
     * Fetch constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \FiloBlu\Flow\Model\Processor $_processor
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Processor $_processor
        ) {
        $this->_logger = $loggerProvider->getLogger();
        $this->_processor = $_processor;
    }

    public function __destruct() {
        // Turned off, missing mysql user permissions
        //$this->resetWaitMysqlVariables();
    }

    public function execute() {
        $this->_logger->info(__METHOD__);
        $this->_processor->fetchChannels();
        return $this;
    }

    public function executeBus1() {
        $this->_logger->info(__METHOD__);
        $this->_processor->fetchChannels(1);
        return $this;
    }

    public function executeBus2() {
        $this->_logger->info(__METHOD__);
        $this->_processor->fetchChannels(2);
        return $this;
    }

    public function executeBus3() {
        $this->_logger->info(__METHOD__);
        $this->_processor->fetchChannels(3);
        return $this;
    }

    public function executeBus4() {
        $this->_logger->info(__METHOD__);
        $this->_processor->fetchChannels(4);
        return $this;
    }

    public function receive() {
        $this->_logger->info(__METHOD__);
        $this->_processor->receiveData();
        return $this;
    }

    /**
     * @return $this
     * @throws \Exception
     */
    public function parse() {
        // Turned off, missing mysql user permissions
        //$this->setWaitMysqlVariables();
        $this->_logger->info(__METHOD__);
        $this->_processor->parse();
        return $this;
    }

    /**
     * @return $this
     * @throws \Exception
     */
    public function process() {
        $this->_logger->info(__METHOD__);
        $this->_processor->process();
        return $this;
    }

    /**
     * @return $this
     * @throws \Exception
     */
    public function processCritical() {
        $this->_logger->info(__METHOD__);
        $this->_processor->processCritical();
        return $this;
    }

    /**
     * @return $this
     * @throws \Exception
     */
    public function clean() {
        $this->_logger->info(__METHOD__);
        $this->_processor->clean();
        return $this;
    }

    /*
     * This function will change the MYSQL Variables
     * - wait_timeout
     * - interactive_timeout
     * to prevent 'General error: 2006 MySQL server has gone away ' error.
     */
    public function setWaitMysqlVariables(){
        $resource = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);

        //Reading current values
        $result_wait_timeout = $connection->fetchAll("SHOW GLOBAL VARIABLES LIKE 'wait_timeout'");
        if(isset($result_wait_timeout[0]['Value'])){
            // Saving old value
            $this->old_wait_timeout = $result_wait_timeout[0]['Value'];
            // Changing it
            $connection->query("SET GLOBAL wait_timeout={$this->new_wait_value_timeout}");
        }
        $result_interactive_timeout = $connection->fetchAll("SHOW GLOBAL VARIABLES LIKE 'interactive_timeout'");
        //Reading current values
        if(isset($result_interactive_timeout[0]['Value'])){
            // Saving old value
            $this->old_interactive_timeout = $result_interactive_timeout[0]['Value'];
            // Changing it
            $connection->query("SET GLOBAL interactive_timeout={$this->new_wait_value_timeout}");
        }
    }

    /*
     * This function will rollback the MYSQL Variables
     * - wait_timeout
     * - interactive_timeout
     * to the old value (if there was one)
     */
    public function resetWaitMysqlVariables(){
        $resource = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);

        if(isset($this->old_wait_timeout) && $this->old_wait_timeout > 0){
            $connection->query("SET GLOBAL interactive_timeout={$this->old_wait_timeout}");
        }
        if(isset($this->old_interactive_timeout) && $this->old_interactive_timeout > 0){
            $connection->query("SET GLOBAL interactive_timeout={$this->old_interactive_timeout}");
        }
    }
}
