<?php
namespace FiloBlu\Flow\Model\From;

use FiloBlu\Flow\Model\ResourceModel\Inboundflow\Collection;

/**
 * Class Product
 * Used in process of Item (Products)
 * @package FiloBlu\Flow\Model\From
 */
class Product extends AbstractFrom {

    /**
     * @var \Monolog\Logger
     */
    protected $_logger;
    protected $_context;
    protected $_scopeConfig;
    protected $_attributeSets = [];
    const CONFIG_FLOW_WEBSITES_MAPPING = 'filoblu_flow/flow_ite/masterfile_website_mapping';

    /**
     * Product constructor.
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface           $scopeConfig
     * @param \Magento\Framework\Model\Context                             $context
     * @param \Magento\Framework\Registry                                  $registry
     * @param \FiloBlu\Flow\Helper\Product                                 $helper_product
     * @param \FiloBlu\Flow\Helper\Data                                    $helper_flow
     * @param \FiloBlu\Flow\Helper\Attribute                               $helper_attribute
     * @param \FiloBlu\Flow\Helper\LoggerProvider                          $loggerProvider
     * @param \Magento\Store\Model\StoreManagerInterface                   $storeManager
     * @param \Magento\Catalog\Model\ProductFactory                        $productFactory
     * @param \Magento\Eav\Api\AttributeSetRepositoryInterface             $attributeSetRepository
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null           $resourceCollection
     * @param array                                                        $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\Attribute $helper_attribute,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Eav\Model\ResourceModel\Entity\Attribute\Set\CollectionFactory $attributeSetCollectionFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->_scopeConfig = $scopeConfig;
        $this->_context = $context;
        $this->helper_product = $helper_product;
        $this->helper_flow = $helper_flow;
        $this->helper_attribute = $helper_attribute;
        $this->storeManager = $storeManager;
        $this->productFactory = $productFactory;
        $this->_logger = $loggerProvider->getLogger();
        $this->getProductAttributeSetCollection($attributeSetCollectionFactory);
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Product');
    }

    /**
     * Loads all product attribute in an array
     *
     * @param $collection
     */
    public function getProductAttributeSetCollection($collection) {
        $attributeSetCollection = $collection->create()->addFieldToFilter('entity_type_id',4)->toArray();
        foreach($attributeSetCollection['items'] as $item){
            $this->_attributeSets[$item['attribute_set_name']] = $item['attribute_set_id'];
        }
    }

    /**
     * Get Attribute Set From Name
     *
     * @param $name
     *
     * @return int|mixed|null
     */
    public function getProductAttributeSetFromName($name) {
        $setId = 4;
        if(!empty($name)){
            if(array_key_exists($name,$this->_attributeSets)){
                $setId = $this->_attributeSets[$name];
            }
        }
        return $setId;
    }

    /**
     * @return $this
     */
    public function process() {

        $url_trail = '';

        $ProdHelper = $this->helper_product;

        //Get Attribute Dinamically
        $attributeSetId = $this->getProductAttributeSetFromName($this->getAttributeSet());
        if (empty($attributeSetId)) {
            $message = sprintf("Product attribute_set not defined!");
            $this->error(true, $message);
            $this->_logger->info($message);
            return $this;
        }

        $sku_configurabile = $this->getCodiceConfigurabile();

        $TaxClassCode = $this->getFasciaIvaArticoli();
        $mappedTaxClassId = $ProdHelper->getMappedTaxClassId($TaxClassCode);
        if (!$mappedTaxClassId) {
            $message = sprintf("tax_class_code '%s' has no Magento tax class associated.", $TaxClassCode);
            $this->error(true, $message);
            $this->_logger->info($message);
            return $this;
        }

        $productSku = $this->helper_flow->removeTextSpaces($this->getCodiceArticolo());

        if (empty($productSku)) {
            return $this->error(false, "'codice_articolo' is empty.");
        }

        $model_product = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Product');

        $product_id = $model_product->getIdBySku($productSku);

        if (!$product_id) {

            if (!$ProdHelper->createNewProducts()) {
                $message = sprintf('The product with SKU %s has not been found and the module is configured to not create it', $skuFilter);
                $this->error(true, $message);
                $this->_logger->info($message);
                return $this;
            }

            if(strrpos($productSku, '-',-1) !== false)
            {
                $url_trail = 'u';
            }

            $product = $model_product
                ->setTypeId(\Magento\Catalog\Model\Product\Type::TYPE_SIMPLE)
                ->setAttributeSetId($attributeSetId)
                ->setStatus($ProdHelper->getInitialProductStatus())
                ->setVisibility(\Magento\Catalog\Model\Product\Visibility::VISIBILITY_BOTH)
                ->setStoreId(0);

            //Check if the name need to be copied on Description
            if($this->helper_product->getCopyNameInDescription()){
                $model_product->setDescription($this->getNomeArticolo());
            }

            //Check if the name need to be copied on Short Description
            if($this->helper_product->getCopyNameInShortDescription()){
                $model_product->setShortDescription($this->getNomeArticolo());
            }

            //Check if Disable Module Shop is enabled
            if($this->helper_product->isDisableShopModuleEnabled()){
                $disableShopStatus = 0;
                $hidePriceStatus = 0;
                if($this->helper_product->getDisableShopStatus()) {
                    $disableShopStatus = 1;
                }
                if($this->helper_product->getHidePriceStatus()) {
                    $hidePriceStatus = 1;
                }
                $model_product->setDisableShop($disableShopStatus);
                $model_product->setHidePrice($hidePriceStatus);
            }

            $model_product->setSku($productSku)
                ->setUrlKey(strtolower(preg_replace('#[^0-9a-z]+#i', '-', $this->getNomeArticolo().'-'.$productSku)) . $url_trail)
                ->setPrice(0)
                ->setPriceView(0)
                ->setWeight(0)
                ->setName($this->getNomeArticolo());


            if ($ProdHelper->getAddToWebsites()) {

                $websites = $this->storeManager->getWebsites();

                foreach ($websites as $website) {
                    $websitesIds[] = $website->getId();
                }

                $product->setWebsiteIds($websitesIds);
            }
            else {
                $current_file_id = $this->getMetaFile();
                $inbound_file = $this->helper_flow->getChannelByInbound($current_file_id);
                $current_channel = $inbound_file['channel'];
                $config_mapping = unserialize($this->_scopeConfig->getValue(self::CONFIG_FLOW_WEBSITES_MAPPING));
                $websitesIds = array();
                foreach ($config_mapping as $row_id => $config_row) {
                    if(trim($config_row['channel']) == trim($current_channel))
                        $websitesIds = $config_row['website_id'];
                }

                $product->setWebsiteIds($websitesIds);
            }

        }

        $productAttributesData = array(
            "product_sku" => $this->getCodiceArticolo(),
            "tax_class_id" => $mappedTaxClassId,
            "parent_sku" => $sku_configurabile
        );


        //Setting configurable attributes on simple (if there are any)
        $configurable_attributes = array_filter(explode(",",$this->getConfigurableAttributes()));
        $configurable_values = array_filter(explode(",",$this->getConfigurableValues()));
        $create_configurable = false;
        if(count($configurable_attributes) > 0 && count($configurable_values) == count($configurable_attributes)){
            $configurableAttributesData = array_combine($configurable_attributes, $configurable_values);
            foreach ($configurableAttributesData as $code => $label) {
                $optionId = $this->helper_attribute->createOrGetId($code, $label);
                $productAttributesData = array_merge($productAttributesData, array($code => $optionId));
            }
            $create_configurable = true;
        }


        //initial stock configuration
        $stockData = $this->getInitialStockData();

        if (!$product_id) {
            //setting the initial stock config...
            $productAttributesData["stock_data"] = $stockData;
            //saving the product...
            $product->addData($productAttributesData);
            try{
                $product->save();
            }catch(\Exception $e){
                $message = $e->getMessage();
                $this->_logger->info($message);
            }

        } else {

            // da implementare

            //if (true or !$this->getVisibilitaSuStore()) {
            //$productAttributesData["status"] = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED;
            //}

            //saving attributes...(ERRORE IN UPDATE PRODOTTO)
            //\Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Product\Action')->updateAttributes(array($product_id), $productAttributesData, 0);

            //saving stock configuration...
            /*
            $stockItem = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\CatalogInventory\Api\StockRegistryInterface')->getStockItem($product_id,0);

            if ( $product_id != \Magento\Catalog\Model\Product\Type::TYPE_BUNDLE && ( $stockData['backorders'] != \Magento\CatalogInventory\Model\Stock::BACKORDERS_NO &&
                    ($stockData['min_qty'] == 0 || $stockItem->getQty() > $stockData['min_qty'])
                ) ) {
                $stockItem->setIsInStock(true);
            } elseif( $product_id != \Magento\Catalog\Model\Product\Type::TYPE_BUNDLE ) {
                if($stockItem->getQty() >= $stockItem->getMinSaleQty()){
                    $stockItem->setIsInStock(true);
                }else{
                    $stockItem->setIsInStock(false);
                }
            }

            $stockItem->addData($stockData)->save();
            */

        }

        $className = "\FiloBlu\Flow\Model\Actions\From" . "\\" . ucfirst(str_replace("_","","_add_to_configurable"));

        // Need the product to create the configurable!
        if($create_configurable && !isset($product)){
            $product = $this->productFactory->create()->load($product_id);
        }

        if($create_configurable && isset($product) && $product->getId() && class_exists($className)){
            $specificSpecialClass = \Magento\Framework\App\ObjectManager::getInstance()->get($className);
            // Handling multiple parents for a single simple
            $configurable_skus = $this->getCodiceConfigurabile();
            $configurable_skus_array = explode(",",$configurable_skus);
            foreach ($configurable_skus_array as $single_configurable_sku) {
                $single_configurable_sku = trim($single_configurable_sku);
                $value = $single_configurable_sku."#".implode(",",$configurable_attributes);
                $result = $specificSpecialClass->process($product, $value, 0, $this->getVisibility());
            }

        }

        return $this;
    }

    public function getInitialProductStatus() {
        $h = Mage::helper('fbflow/product');
        $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_DISABLED;
        if ($h->useSupplierProductStatus()) {
            if (!$this->getVisibilitaSuStore()) {
                $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_DISABLED;
            } else {
                $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED;
            }
        } else {
            $status = $h->getDefaultProductStatus();
        }

        return $status;
    }

    public function getInitialStockData() {
        $stockData = array(
            'qty' => 0,
            'is_in_stock' => 0
        );
        $stockData["use_config_min_qty"] = true;
        return $stockData;
    }




}